# Combine.tcl: combines all output files into database file
# Units: kip, in
# Kevin Mackie, 2001/02/01
# mackie@ce.berkeley.edu
#
# Takes ground motion and structural input parameters from higher
# level procedure and outputs UCSD output quantities.

proc Combine {param_group directory sf L LoH ida Treturn} {

wipe

set g 386.4
set pi [expr acos(-1.0)]
set Ht [expr $L/$LoH]
set ida_cnt 1

# Extract period information from input variable Treturn
scan $Treturn "%f %f %f %f" T1 T2 Td1 Td2

# -----------------------------------------------------
# set up earthquake files and parameters

set indxfile [open [format "%sIndex.tcl" $directory] "r"]
set total_eqks 0
while {-1 != [gets $indxfile line] } {
   set total_eqks [expr $total_eqks+1]
   scan $line "%s %s %s" eqk_fp($total_eqks) eqk_fn($total_eqks) eqk_fv($total_eqks) 
}
close $indxfile

# -----------------------------------------------
# calculate R (or B) factor for bridge
# using averaged Sa(T=bridge period) from bin
# like in Capacity spectrum method
# Note: this should be compared to T=1 average method
set Sa1_avgl 0.0
set Sa1_totl 0
set Sa1_avgt 0.0
set Sa1_tott 0
set imcdm "/home/mackie/motion/spectral"

for {set loop_var 1} {$loop_var<=$total_eqks} {incr loop_var} {
   set fp [format "%s%s.AT2" $directory $eqk_fp($loop_var)]
   set fn [format "%s%s.AT2" $directory $eqk_fn($loop_var)]

   set SaT1csp1 [exec $imcdm $fn $T2 0.05 $sf]
   set SaT1csp2 [exec $imcdm $fp $T1 0.05 $sf]

   # actually spits out Sd, not Sa, so convert to Sa below
   set Sa1_avgt [expr $Sa1_avgt + 4.0*pow($pi,2)/pow($T2,2)*$SaT1csp1]
   incr Sa1_tott
   set Sa1_avgl [expr $Sa1_avgl + 4.0*pow($pi,2)/pow($T1,2)*$SaT1csp2]
   incr Sa1_totl

   # This is residual from old cdm versions, delete the file
   set fstubSA [format "f%04i" $loop_var]
   file delete [format "%ssum_motion.out" $fstubSA]
}
# these quantities are all in cm/s^2 from spectral
set Rfl [expr $Sa1_avgl/$Sa1_totl]
set Rft [expr $Sa1_avgt/$Sa1_tott]

# -----------------------------------------------
# calculate R factor for bridge
# using USGS 2% in 50 year spectrum
source usgs.tcl
set spec2 [usgs "../motion/usgs.txt"]
scan $spec2 "%f %f %f" sa1 sa2 sa3
set gmetric 980.6
set sa1 [expr $sa1/100*$gmetric]
set sa2 [expr $sa2/100*$gmetric]
set sa3 [expr $sa3/100*$gmetric]
set Tstar [expr $sa3/$sa1]
if { $T1 > $Tstar } {
   set Rful [expr $sa3/$T1]
} else {
   set Rful $sa1
}
if { $T2 > $Tstar } {
   set Rfut [expr $sa3/$T2]
} else {
   set Rfut $sa1
}

# -----------------------------------------------------
# begin loop of analysis for each ground motion triplet above
for {set loop_var 1} {$loop_var<=$total_eqks} {incr loop_var} {

# ground motion data for this run (they must have corresponding dt vals)
set fp [format "%s%s.AT2" $directory $eqk_fp($loop_var)]
set fn [format "%s%s.AT2" $directory $eqk_fn($loop_var)]
set fv [format "%s%s.AT2" $directory $eqk_fv($loop_var)]

set ida_stop 0
while { $ida_stop == 0 } {
   # ground motion data for this run (they must have corresponding dt vals)
   set sfilen [format "fs%04i.out" $param_group]

   if { $ida == 1 } {
      set dfilen [format "fd%04i%04i%02i.out" $param_group $loop_var $ida_cnt]
      set tfilen [format "f%04i%04i%02i.out" $param_group $loop_var $ida_cnt]
   } else {
      set dfilen [format "fd%04i%04i.out" $param_group $loop_var]
      set tfilen [format "f%04i%04i.out" $param_group $loop_var]
      set ida_stop 1
   }

   set filet [open $tfilen "w"]

   # -----------------------------------------------------
   # Append bridge specific static analysis quantities computed earlier 
   # using AnalysisS.tcl
   set filest [open $sfilen "r"]
   set filedy [open $dfilen "r"]
   while {-1 != [gets $filest line] } {
      puts $filet $line
      set parser [split $line]
      set parsel [llength $parser]
      set static_text ""
      for { set kl 0 } { $kl < $parsel } { incr kl } {
         if { [string is double [lindex $parser $kl]] } {
            set static_quantity [lindex $parser $kl]
         } else {
            set static_text [concat $static_text [lindex $parser $kl]]
         }
      }
      if { [string first $static_text "Yield disp. long."] >= 0 } {
         set uy $static_quantity
      }
      if { [string first $static_text "Yield disp. trans."] >= 0 } {
         set uyt $static_quantity
      }
      if { [string first $static_text "Yield curv. long."] >= 0 } {
         set ky $static_quantity
      }
      if { [string first $static_text "Yield curv. trans."] >= 0 } {
         set kyt $static_quantity
      }
      if { [string first $static_text "Capacity Cy long."] >= 0 } {
         set Cyl $static_quantity
      }
      if { [string first $static_text "Capacity Cy trans."] >= 0 } {
         set Cyt $static_quantity
      }
      if { [string first $static_text "Yield Energy long."] >= 0 } {
         set ey $static_quantity
      }
      if { [string first $static_text "Yield Energy trans."] >= 0 } {
         set eyt $static_quantity
      }
   }
   close $filest

   # -----------------------------------------------------
   # Compute all R factors

   #set Rfacl [expr $Rfac/$Cyl/2.0/$pi*sqrt($Cyl*$g/$uy)]
   #set Rfact [expr $Rfac/$Cyt/2.0/$pi*sqrt($Cyt*$g/$uyt)]
   # Note, all R factors are in cm/s^2 whereas, Cy factors
   # are all in in/s^2
   set Rfacl [expr $Rfl/$Cyl/2.54]
   set Rfact [expr $Rft/$Cyt/2.54]
   
   set Rfaclu [expr $Rful/$Cyl/2.54]
   set Rfactu [expr $Rfut/$Cyt/2.54]

   # -----------------------------------------------------
   # Append bridge specific dynamic analysis quantities computed earlier 
   # using AnalysisD.tcl
   while {-1 != [gets $filedy line] } {
      puts $filet $line
      set parser [split $line]
      set parsel [llength $parser]
      set static_text ""
      for { set kl 0 } { $kl < $parsel } { incr kl } {
         if { [string is double [lindex $parser $kl]] } {
            set static_quantity [lindex $parser $kl]
         } else {
            set static_text [concat $static_text [lindex $parser $kl]]
         }
      }
      if { [string first $static_text "Max_displ. in x"] >= 0 } {
         set max_x $static_quantity
      }
      if { [string first $static_text "Max_displ. in y"] >= 0 } {
         set max_y $static_quantity
      }
      if { [string first $static_text "Max_curv in long."] >= 0 } {
         set max_kx $static_quantity
      }
      if { [string first $static_text "Max_curv in trans."] >= 0 } {
         set max_ky $static_quantity
      }
      if { [string first $static_text "residual in x"] >= 0 } {
         set resid_x $static_quantity
      }
      if { [string first $static_text "residual in y"] >= 0 } {
         set resid_y $static_quantity
      }
      if { [string first $static_text "Hysteretic energy ln."] >= 0 } {
         set E_l $static_quantity
      }
      if { [string first $static_text "Hysteretic energy tr."] >= 0 } {
         set E_t $static_quantity
      }
   }
   close $filedy

   # ------------------------------------------------
   # report residual displacements and calculate RDI
   set RDI_l [expr abs($resid_x)/$uy]
   set RDI_t [expr abs($resid_y)/$uyt]

   # ------------------------------------------------
   # report plastic hinge angle
   set theta_px [expr ($max_x-$uy)/$Ht*100.0]
   set theta_py [expr ($max_y-$uyt)/$Ht*100.0]
   if { $theta_px < 0 } {
      set theta_px 0.000001
   }
   if { $theta_py < 0 } {
      set theta_py 0.000001
   }

   # -----------------------------------------------------
   # compute maximum ductilities
   set mudl [expr $max_x/$uy]
   set mukl [expr $max_kx/$ky]

   set mudt [expr $max_y/$uyt]
   set mukt [expr $max_ky/$kyt]

   # -----------------------------------------------------
   # compute maximum energies
   set NHEl [expr $E_l/$ey]
   set NHEt [expr $E_t/$eyt]

   # -----------------------------------------------------
   # compute quantities with static and dynamic dependencies
   puts $filet "R (cs) long.           $Rfacl"
   puts $filet "R (cs) tran.           $Rfact"
   puts $filet "R (usgs) long.         $Rfaclu"
   puts $filet "R (usgs) tran.         $Rfactu"
   puts $filet "disp. ductility long.  $mudl"
   puts $filet "curv. ductility long.  $mukl"
   puts $filet "disp. ductility trans. $mudt"
   puts $filet "curv. ductility trans. $mukt"
   puts $filet "RDI long.              $RDI_l"
   puts $filet "RDI tran.              $RDI_t"
   puts $filet "theta_px               $theta_px"
   puts $filet "theta_py               $theta_py"
   puts $filet "Normalized HE long.    $NHEl"
   puts $filet "Normalized HE trans.   $NHEt"

   # -----------------------------------------------------
   # Extract other ground motion only dependent intensity measures
   # from .info file (created by program "im")
   # Note, can be as many as are computed in "im"
   #
   # Note: IDA's overwrite IM's in .info files
   # as intensity is increased, make sure to extract
   # old values of IM's from temp files created by ModelD

   if { $ida == 1 } {
      set filed1 [open [format "gmfn%04i%04i%02i.info" $param_group $loop_var $ida_cnt] "r"]
      set filed2 [open [format "gmfp%04i%04i%02i.info" $param_group $loop_var $ida_cnt] "r"]
      set filed3 [open [format "gmfv%04i%04i%02i.info" $param_group $loop_var $ida_cnt] "r"]
   } else {
      set filed1 [open [format "gmfn%04i%04i.info" $param_group $loop_var] "r"]
      set filed2 [open [format "gmfp%04i%04i.info" $param_group $loop_var] "r"]
      set filed3 [open [format "gmfv%04i%04i.info" $param_group $loop_var] "r"]
   }
   set line [gets $filed1]
   scan $line "%i %lf %u" points dt eqk_num
   set duration [expr $points*$dt]
   puts $filet "Duration               $duration"
   set line [gets $filed2]
   set line [gets $filed3]
   while {-1 != [gets $filed1 lined1] } {
      puts $filet $lined1
      if { [string first "Magnitude" $lined1] >= 0 } {
         # don't reprint the Magnitude information
         gets $filed2 lined2
         gets $filed3 lined3
      } elseif { [string first "Distance" $lined1] >= 0 } {
         # don't reprint the Distance information
         gets $filed2 lined2
         gets $filed3 lined3
      } elseif { [string first "Strong motion" $lined1] >= 0 } {
         # don't reprint the strong motion duration information
         gets $filed2 lined2
         gets $filed3 lined3
      } elseif { [string first "Vibration periods" $lined1] >= 0 } {
         # don't reprint the period information
         gets $filed2 lined2
         gets $filed3 lined3
      } elseif { [string first "Damaged periods" $lined1] >= 0 } {
         # don't reprint the damaged period information
         gets $filed2 lined2
         gets $filed3 lined3
      } else {
         gets $filed2 lined2
         puts $filet $lined2
         gets $filed3 lined3
         puts $filet $lined3
      }
   }
   close $filed1
   close $filed2
   close $filed3  

   close $filet

   # delete analysis files
   #file delete [format "fd%04ie04isum.out" $param_group $eqk_num]
   # delete temp ground motion files
   if { $ida == 1 } {
      file delete [format "gmfn%04i%04i%02i.info" $param_group $loop_var $ida_cnt]
      file delete [format "gmfp%04i%04i%02i.info" $param_group $loop_var $ida_cnt]
      file delete [format "gmfv%04i%04i%02i.info" $param_group $loop_var $ida_cnt]
   } else {
      file delete [format "gmfn%04i%04i.info" $param_group $loop_var]
      file delete [format "gmfp%04i%04i.info" $param_group $loop_var]
      file delete [format "gmfv%04i%04i.info" $param_group $loop_var]
   }

   set ida_cnt [expr $ida_cnt+1]
   set dfilen [format "fd%04i%04i%02i.out" $param_group $loop_var $ida_cnt]
   if { [file exists $dfilen] == 0} {
      set ida_stop 1
   }
   # while $ida_stop == 0 ends here
}  

set ida_cnt 1
# enter arguments before end of for loop here
}
}
