# ModelD.tcl: generic input shell for use with dynamic models
# Units: kip, in
# Kevin Mackie, 2001/08/01
# mackie@ce.berkeley.edu
#
# Takes ground motion and structural input parameters from higher
# level procedure and outputs data to files for computation of UCSD
# output quantities.  Performs dynamic excitation only

proc ModelD {param_group directory sf skew L LoH fy fc rho_s DcDs ksoil wt rhos_trans deck abut ida} {

wipe

set g 386.4
set IDAMIN 0.50
set IDAMAX 3.70
set ida_incr 0.80

# -----------------------------------------------------
# set up earthquake files and parameters

set indxfile [open [format "%sIndex.tcl" $directory] "r"]
set total_eqks 0
while {-1 != [gets $indxfile line] } {
   set total_eqks [expr $total_eqks+1]
   scan $line "%s %s %s" eqk_fp($total_eqks) eqk_fn($total_eqks) eqk_fv($total_eqks) 
}
close $indxfile

# -----------------------------------------------------
# begin loop of analysis for each ground motion triplet above
for {set loop_var 1} {$loop_var<=$total_eqks} {incr loop_var} {

puts "Parameter group $param_group: EQ $eqk_fv($loop_var)"

# ground motion data for this run (they must have corresponding dt vals)
set fp [format "%s%s.AT2" $directory $eqk_fp($loop_var)]
set fn [format "%s%s.AT2" $directory $eqk_fn($loop_var)]
set fv [format "%s%s.AT2" $directory $eqk_fv($loop_var)]

# -----------------------------------------------------
# iterate with increasing intensity if ida=1

set ida_stop 0
set ida_cnt 1
set ida_level $IDAMIN
source Transient.tcl
if { $ida == 1 } {
   # perform ida before moving onto next record
   while { $ida_stop==0 } {
      puts "   Intensity level: $ida_level"
      set Tret [Transient $param_group $skew $L $LoH $fy $fc $rho_s $DcDs $ksoil $wt $rhos_trans $deck $abut $ida $ida_level $ida_cnt $fn $fp $fv]

      # copy ground motion *.info file to a temp file for use by
      # Combine.tcl otherwise incremental info will be overwritten
      file copy -force [format "%s.info" $fn] [format "gmfn%04i%04i%02i.info" $param_group $loop_var $ida_cnt]
      file copy -force [format "%s.info" $fp] [format "gmfp%04i%04i%02i.info" $param_group $loop_var $ida_cnt]
      file copy -force [format "%s.info" $fv] [format "gmfv%04i%04i%02i.info" $param_group $loop_var $ida_cnt]

      # future: adjust ida_incr here based on results of last run
      # ie, if we wanted to monitor some IM or DM quantity instead
      # of straight scaling

      set ida_level [expr $ida_level+$ida_incr]
      set ida_cnt [expr $ida_cnt + 1]
      if { $ida_level > $IDAMAX } {
         set ida_stop 1
      }
   }
} else {
   # just perform a single transient analysis for this record
   # all record scaled by factor of sf (2 usually)
   set Tret [Transient $param_group $skew $L $LoH $fy $fc $rho_s $DcDs $ksoil $wt $rhos_trans $deck $abut $ida $sf $ida_cnt $fn $fp $fv]

   # copy ground motion *.info file to a temp file for use by Combine.tcl
   file copy -force [format "%s.info" $fn] [format "gmfn%04i%04i.info" $param_group $loop_var]
   file copy -force [format "%s.info" $fp] [format "gmfp%04i%04i.info" $param_group $loop_var]
   file copy -force [format "%s.info" $fv] [format "gmfv%04i%04i.info" $param_group $loop_var]
}

}
return $Tret
}
