# Transient.tcl: sets up and runs arbitrary transient run
# Units: kip, in
# Kevin Mackie, 2001/06/06
# mackie@ce.berkeley.edu
#
# Takes all parameters from ModelD.tcl and runs transient analysis.
# Allows multiple runs on same record as required for IDA
# Performs Earthquake excitation transient analysis

proc Transient {param_group skew L LoH fy fc rho_s DcDs ksoil wt rhos_trans deck abut ida level ida_cnt fn fp fv} {

wipe

set gap_in 6.0
set g 386.4
set pi [expr acos(-1.0)]
set damping_ratio 0.02

# load ground motion data parameters
set file [open [format "%s.info" $fn] "r"]
set line [gets $file]
scan $line "%i %lf %u" points dt eqk_num
close $file
set filen [format "%s.data" $fn]
set filep [format "%s.data" $fp]
set filev [format "%s.data" $fv]

# -----------------------------------------------------
# Perform eigen analysis (initial stiffness)

source GenerateBridge.tcl
GenerateBridge $skew $L $LoH $fy $fc $rho_s $DcDs $ksoil $wt $rhos_trans $deck $gap_in $abut

# perform eigenvalue analysis for first two modes
set eigenvalues [eigen 5] 
set omega1 [lindex $eigenvalues 0]
set omega2 [lindex $eigenvalues 1]

# -----------------------------------------------------
# Perform Earthquake excitation transient analysis

wipe

GenerateBridge $skew $L $LoH $fy $fc $rho_s $DcDs $ksoil $wt $rhos_trans $deck $gap_in $abut

if { $ida == 1 } {
   set fstub [format "d%04i%04i%02i" $param_group $eqk_num $ida_cnt]
} else {
   set fstub [format "d%04i%04i" $param_group $eqk_num]
}

# Create a recorder which writes to output
recorder Node [format "%snd08.out" $fstub] disp -time -node  8 -dof 1 2 3

# Record section deformations at the left column
recorder Element 1 -file [format "%ssf011.out" $fstub] section 1 forces
recorder Element 1 -file [format "%ssd011.out" $fstub] section 1 deformations
recorder Element 1 -file [format "%ssf015.out" $fstub] section 5 forces
recorder Element 1 -file [format "%ssd015.out" $fstub] section 5 deformations

# Record corresponding rotations for energy calculations
recorder Node [format "%snr011.out" $fstub] disp -node  1 -dof 4 5 6
recorder Node [format "%snr015.out" $fstub] disp -node  8 -dof 4 5 6

# Record left gap properties
#recorder Element 40 -file [format "%sef401.out" $fstub] forces
#recorder Element 40 -file [format "%sed401.out" $fstub] deformations
#recorder Element 41 -file [format "%sef411.out" $fstub] forces
#recorder Element 41 -file [format "%sed411.out" $fstub] deformations

# Record right gap properties
#recorder Element 42 -file [format "%sef421.out" $fstub] forces
#recorder Element 42 -file [format "%sed421.out" $fstub] deformations
#recorder Element 43 -file [format "%sef431.out" $fstub] forces
#recorder Element 43 -file [format "%sed431.out" $fstub] deformations

# Record fiber stress and strain at max location
set mat_file [open "fiber_numbers" "r"]
gets $mat_file line
scan $line "%i %i" conc_fibers steel_fibers
close $mat_file

for {set i 20} {$i < $conc_fibers} {incr i} {
recorder Element 1 -file [format "%sffc%03i.out" $fstub $i] section 5 fiber $i stress
recorder Element 1 -file [format "%sfdc%03i.out" $fstub $i] section 5 fiber $i strain
}

# Record fiber stress and strain at max location
for {set i $conc_fibers} {$i < $conc_fibers+$steel_fibers} {incr i} {
recorder Element 1 -file [format "%sffs%03i.out" $fstub $i] section 5 fiber $i stress
recorder Element 1 -file [format "%sfds%03i.out" $fstub $i] section 5 fiber $i strain
}

# Use same integrator, test, algorithms, etc as pushovers
# for initial gravity load analysis
#                     init Jd  min  max
integrator LoadControl  0   1   0    0
#                  tolerance maxIter displayCode
test EnergyIncr     1.0e-12     15        0
algorithm Newton
numberer RCM
constraints Plain
system UmfPack
analysis Static
analyze 1
wipeAnalysis

# reset time in time domain
loadConst -time 0.0

# define earthquakes    filename    dt        factor            
set path1 "Path -filePath $filen -dt $dt -factor [expr $g*$level]"
set path2 "Path -filePath $filep -dt $dt -factor [expr $g*$level]"
set path3 "Path -filePath $filev -dt $dt -factor [expr $g*$level]"

#                         tag dir 
pattern UniformExcitation  2   1  -accel $path1
pattern UniformExcitation  3   2  -accel $path2
pattern UniformExcitation  4   3  -accel $path3

set omega1 [expr sqrt($omega1)]
set omega2 [expr sqrt($omega2)]
set alphaM [expr $damping_ratio*2.0*$omega1*$omega2/($omega1+$omega2)]
set betaK [expr $damping_ratio*2.0/($omega1+$omega2)]

# Rayeigh damping for first 2 modes assuming 2% viscous damping
# C = alphaM * M + betaK * Kcurrent + betaKi * Kinit + betaKc * Kcommit
#                  gamma beta  alphaM       betaK
integrator Newmark  0.5  0.25  $alphaM   0.0  0.0  $betaK

#                  tolerance maxIter displayCode
test EnergyIncr     1.0e-12     15        0
algorithm Newton
numberer RCM
constraints Plain
system UmfPack
initialize
analysis Transient
set res [analyze [expr $points+25] $dt]
if {$res < 0} {
    puts "Dynamic analysis failed\n"
}

# -----------------------------------------------------
# Perform eigen analysis (post earthquake stiffness)

wipeAnalysis

# perform eigenvalue analysis for first two modes
set eigenvalues [eigen 5] 
set omega1dmg [lindex $eigenvalues 0]
set omega2dmg [lindex $eigenvalues 1]
if {$omega1dmg > 0} {
   set omega1dmg [expr sqrt($omega1dmg)]
} else {
   set omega1dmg 0
}
if {$omega2dmg > 0} {
   set omega2dmg [expr sqrt($omega2dmg)]
} else {
   set omega2dmg 0
}
wipe

# ------------------------------------------------

source AnalysisD.tcl
AnalysisD $skew $L $LoH $fy $fc $rho_s $DcDs $ksoil $wt $rhos_trans $eqk_num $param_group $ida $ida_cnt

# append ground motion specific data, ie Sd, Sa, etc
# using program cdm which uses central difference to compute
# spectral response quantities, generates information which is
# added to ground motion .info files
set Tn1 [expr 2.0*$pi/$omega1]
set Tn2 [expr 2.0*$pi/$omega2]
if { $omega1dmg <= 0 } {
   set Tn1dmg 0.01
} else {
   set Tn1dmg [expr 2.0*$pi/$omega1dmg]
}
if { $omega2dmg <= 0 } {
   set Tn2dmg 0.01
} else {
   set Tn2dmg [expr 2.0*$pi/$omega2dmg]
}

set Treturn "$Tn1 $Tn2 $Tn1dmg $Tn2dmg"

set imanal "/home/mackie/motion/imcdm"
# note fp in 2 dir : fn in 1 dir : fv in 3 dir
exec $imanal $fn 1 $Tn1 $Tn2 $level $Tn1dmg $Tn2dmg
exec $imanal $fp 2 $Tn1 $Tn2 $level $Tn1dmg $Tn2dmg
exec $imanal $fv 3 $Tn1 $Tn2 $level $Tn1dmg $Tn2dmg

wipe

# end of analysis
# ------------------------------------------------

# ------------------------------------------------
# remove dynamic recorder files to save space
# leave static files currently to check whether algorithm captured yield or not
if { $ida == 1 } {
   set fstub [format "d%04i%04i%02i" $param_group $eqk_num $ida_cnt]
} else {
   set fstub [format "d%04i%04i" $param_group $eqk_num]
}

file delete [format "%snd08.out" $fstub]
file delete [format "%snr011.out" $fstub]
file delete [format "%snr015.out" $fstub]

#file delete [format "%sef401.out" $fstub]
#file delete [format "%sed401.out" $fstub]
#file delete [format "%sef411.out" $fstub]
#file delete [format "%sed411.out" $fstub]
#file delete [format "%sef421.out" $fstub]
#file delete [format "%sed421.out" $fstub]
#file delete [format "%sef431.out" $fstub]
#file delete [format "%sed431.out" $fstub]

file delete [format "%ssf011.out" $fstub]
file delete [format "%ssd011.out" $fstub]
file delete [format "%ssf015.out" $fstub]
file delete [format "%ssd015.out" $fstub]

return $Treturn

}
